/* eslint-disable @typescript-eslint/no-non-null-assertion */
import { NetworkUnreachableError, InsufficientFundsError, InvalidArgumentError, MissingArgumentError, UnexpectedArgumentError, UnknownBlockchainError, GasTooLowError, InvalidAddressError, ExecutionRevertedError, InvalidNonceError, BlockchainProviderError, } from "@snickerdoodlelabs/objects";
export class BlockchainErrorMapper {
    static buildErrorFromProviderError(chain, error) {
        return new BlockchainProviderError(chain, error.message, error);
    }
    static buildBlockchainError(error, generateGenericError) {
        // First check if error message is a ProviderError as some errors are triggered at the provider level rather than contract level
        try {
            let providerError;
            // if (error?.name == "ProviderError") {
            //   providerError = this.getSpecificProviderError(error);
            // }
            // Get the deepest error in the chain, that should have the most reliable error
            const deepestError = BlockchainErrorMapper.getDeepestError(error);
            const deepestTransaction = BlockchainErrorMapper.getDeepestTransaction(error);
            // Determining the error message is a bit tricky, as it can be in different places depending on the error type
            let errorMessage = "Generic error message, no better message found";
            try {
                if (deepestError.body != null) {
                    const body = JSON.parse(deepestError.body);
                    errorMessage = body.error.message;
                }
                else {
                    errorMessage = deepestError.message;
                }
            }
            catch (e) {
                // Can't parse the body
                errorMessage = deepestError.message;
            }
            const errorInitializerFromMessage = this.blockchainErrorMapping.get(this.getErrorKey(errorMessage));
            if (errorInitializerFromMessage != null) {
                return errorInitializerFromMessage(errorMessage, deepestTransaction, error);
            }
            // If all above are null, then we don't know what the error is
            if (generateGenericError != null) {
                return generateGenericError(errorMessage, error, deepestTransaction);
            }
            return new UnknownBlockchainError(errorMessage, deepestTransaction);
        }
        catch (e) {
            return new UnknownBlockchainError(`Error occurred while running buildBlockchainError ${e}`, e);
        }
    }
    static getErrorKey(errorString) {
        errorString = errorString?.toLowerCase() || "";
        Array.from(this.blockchainErrorMapping.keys()).forEach((key) => {
            if (errorString?.includes(key.toLowerCase())) {
                errorString = key;
            }
        });
        return errorString;
    }
    static getSpecificProviderError(error) {
        // ProviderError has the following properties : [ 'parent', 'name', '_stack', 'code', '_isProviderError', 'data' ]
        // The error message is within the '_stack' property
        // Search for a specific error text and return it if found,
        if (error._stack.includes("intrinsic gas too low")) {
            return "intrinsic gas too low";
        }
        // If no match is found, .includes() will return false, we do not recognize the type ProviderError and return null so that it can be captured as a generic error
        return null;
    }
    // Used to defeat .toString() on Error objects and get the actual goods.
    static printError(error) {
        console.log("Printing error data");
        console.log("message:", error.message);
        console.log("code:", error.code);
        console.log("body:", error.body);
        console.log("error:", error.error == null ? "No Error" : "Deeper Error");
        console.log("requestMethod:", error.requestMethod);
        console.log("url:", error.url);
        console.log("transaction:", error.transaction);
    }
    // This is a recursive function that will return the deepest error in the error chain
    // or itself if it is the deepest error
    static getDeepestError(error) {
        if (error.error == null) {
            return error;
        }
        return BlockchainErrorMapper.getDeepestError(error.error);
    }
    static getDeepestTransaction(error, currentTransaction) {
        // If the current error has a transaction, then it's deeper than the current one.
        if (error.transaction != null) {
            // If there's nothing deeper, we have the transaction
            if (error.error == null) {
                return error.transaction;
            }
            // We have a transaction, but there's a deeper error to check
            return BlockchainErrorMapper.getDeepestTransaction(error.error, error.transaction);
        }
        // No current transaction, but check if there's a deeper error to check
        if (error.error != null) {
            return BlockchainErrorMapper.getDeepestTransaction(error.error, currentTransaction);
        }
        // Nothing deeper to check, no current transaction, return whatever we were given
        return currentTransaction;
    }
}
BlockchainErrorMapper.blockchainErrorMapping = new Map([
    [
        "could not detect network",
        (message, transaction, srcErr) => new NetworkUnreachableError("Could not detect network", srcErr),
    ],
    [
        "insufficient funds for intrinsic transaction cost",
        (message, transaction, srcErr) => new InsufficientFundsError(`Insufficient funds to send transaction: ${message}`, srcErr, transaction),
    ],
    [
        "sender doesn't have enough funds to send tx",
        (message, transaction, srcErr) => new InsufficientFundsError(`Insufficient funds to send transaction: ${message}`, srcErr, transaction),
    ],
    [
        "resolver or addr is not configured for ENS name",
        (message, transaction, srcErr) => new InvalidArgumentError(`Address argument is invalid (format or type): ${message}`, srcErr, transaction),
    ],
    [
        "invalid arrayify value",
        (message, transaction, srcErr) => new InvalidArgumentError(message, srcErr, transaction),
    ],
    [
        "missing argument: passed to contract",
        (message, transaction, srcErr) => new MissingArgumentError(`Missing arguments for contract function call: ${message}`, srcErr, transaction),
    ],
    [
        "too many arguments: passed to contract",
        (message, transaction, srcErr) => new UnexpectedArgumentError(`Too many arguments for contract function call: ${message}`, srcErr, transaction),
    ],
    [
        "intrinsic gas too low",
        (message, transaction, srcErr) => new GasTooLowError(`Insufficient gas provided to function call: ${message}`, srcErr, transaction),
    ],
    [
        "contract creation code storage out of gas",
        (message, transaction, srcErr) => {
            return new GasTooLowError(`Insufficient gas provided to function call: ${message}`, srcErr, transaction);
        },
    ],
    [
        "invalid address",
        (message, transaction, srcErr) => new InvalidAddressError(`Invalid address provided as argument to create contract instance: ${message}`, srcErr, transaction),
    ],
    [
        "execution reverted",
        (message, transaction, srcErr) => new ExecutionRevertedError(`Function hits a revert message on the contract: ${message}`, srcErr, transaction),
    ],
    [
        "nonce has already been used",
        (message, transaction, srcErr) => {
            return new InvalidNonceError(`Nonce has already been used: ${message}`, srcErr, transaction);
        },
    ],
    [
        "nonce too high",
        (message, transaction, srcErr) => {
            return new InvalidNonceError(`Nonce too high: ${message}`, srcErr, transaction);
        },
    ],
    [
        "nonce too low",
        (message, transaction, srcErr) => {
            return new InvalidNonceError(`Nonce too low: ${message}`, srcErr, transaction);
        },
    ],
]);
//# sourceMappingURL=BlockchainErrorMapper.js.map